/*
TECNISA - TECNOLOGIA EM INFORMTICA

NOME: APURAC_MULTIPRODUTO

DESCRIO:
   PROCEDIMENTO PARA APURAO DE FATURAMENTO DE CLIENTES EM FILIAIS
   MULT-PRODUTO.

CRIAO:
03/07/2003 - ALESSANDRA CRISTINA

@C:\WIN_V2\SQL\END_CONV\ORACLE\6209_FAISTP_APURMULTPRODUTO.SQL;

--   RAISE_APPLICATION_ERROR(-20501,V_IDTAXAADM, TRUE);
*/

CREATE OR REPLACE PROCEDURE APURAC_MULTPRODUTO(
   P_CDFILIAPUR  IN FILIAL.CDFILIAL%TYPE,
   P_CDFILIPROC  IN FILIAL.CDFILIAL%TYPE,
   P_CDCLIENTE   IN CLIENTE.CDCLIENTE%TYPE,
   P_NRAPURACFAT IN NOTAFISCAL.NRAPURACFAT%TYPE,
   P_DTINIAPUR   IN DATE DEFAULT NULL,
   P_DTFINAPUR   IN DATE DEFAULT NULL,
   P_DATAPROC    IN DATE,
   P_IDNFSAIDA   IN VARCHAR ) AS


   V_CDEMPRESA   EMPRESA.CDEMPRESA%TYPE;
   V_CDSERIE      SERIENF.CDSERIE%TYPE;

   V_CDFILIAPUR   CTRLAPURFAT.CDFILIAPUR%TYPE;

   V_NRLANCESTQ   LANCTOESTOQ.NRLANCESTQ%TYPE;

   V_VRBRUTITEM   NUMBER;
   V_GERANF       VARCHAR2(01);
   V_LANCAMENTO   VARCHAR2(20);

   V_INSCESTA     VARCHAR2(16);
   V_INSCMUNI     INSCRMUNICIP.CDINSCMUNI%TYPE;

   V_MAXITENSNF   NUMBER;
   V_NRITEM       NUMBER;
   V_PRXMRDP      NUMBER;

      /*CURSOR AUXILIAR PARA DELETAR AS NOTAS FISCAIS*/
   CURSOR C_DELETANF IS SELECT CDFILIAL, NRLANCTONF
                        FROM NOTAFISCAL
                        WHERE CDFILIAL    = P_CDFILIPROC AND
                              NRAPURACFAT = P_NRAPURACFAT;

   CURSOR C_NOTAFISCAL IS
      SELECT N.NRINSJURNF, N.NRNOTAFISC, N.CDFILIAL, N.NRLANCTONF,
            COUNT(*) AS NRITENSNOTA
      FROM NOTAFISCAL N, ITEMPRODUTO I
      WHERE N.CDFILIAL     = I.CDFILIAL    AND
            N.NRLANCTONF   = I.NRLANCTONF  AND
            N.CDFILIAPFATU = P_CDFILIPROC  AND
            N.NRAPFATUTNF  = P_NRAPURACFAT
      GROUP BY N.NRINSJURNF, N.NRNOTAFISC, N.CDFILIAL, N.NRLANCTONF;

   CURSOR C_ITEMNOTA IS
      SELECT I.CDPRODUTO, PT.CDPRODESTO, PT.NMPRODUTO,
             VRUNITITEM,  QTITEMPROD, VRDESCITEM
      FROM NOTAFISCAL N,  PRODUTO PT, ITEMPRODUTO I
      WHERE
         (N.CDFILIEMIT    = P_CDFILIAPUR)                    AND
         (N.CDFILIAPFATU  = P_CDFILIPROC)                    AND
         (N.NRAPFATUTNF   =  P_NRAPURACFAT)                  AND
         (N.DTENTRSAID  BETWEEN P_DTINIAPUR AND P_DTFINAPUR) AND
         (I.CDPRODUTO     = PT.CDPRODUTO)                    AND
         (N.CDFILIAL      = I.CDFILIAL )                     AND
         (N.NRLANCTONF    = I.NRLANCTONF)                    AND
         (I.IDCONFORITEM = 'N');
BEGIN

   V_GERANF     := 'S';
   V_NRITEM     := 0;

   /* FILIAL QUE EST SENDO APURADA (P_FILIAL = FILIPROC)*/
   SELECT MAX(CDFILIAPUR)
   INTO V_CDFILIAPUR
   FROM CTRLAPURFAT
   WHERE CDFILIAL    = P_CDFILIPROC   AND
         NRAPURACFAT = P_NRAPURACFAT;

   IF P_IDNFSAIDA = 'N' THEN
      UPDATE  NOTAFISCAL
      SET CDFILIAPFATU  = P_CDFILIPROC,
          NRAPFATUTNF   = P_NRAPURACFAT
      WHERE  (CDFILIAL, NRLANCTONF) IN (SELECT NF.CDFILIAL, NF.NRLANCTONF
                                 FROM NOTAFISCAL NF, ITEMPRODUTO IP
                                 WHERE (NF.CDFILIAL     = P_CDFILIPROC)                      AND
                                       (NF.CDFILIEMIT   = P_CDFILIAPUR)                      AND
                                       (NF.DTENTRSAID  BETWEEN P_DTINIAPUR AND P_DTFINAPUR)  AND
                                       (NF.IDENTRSAIDOP = 'S')                               AND
                                       (NF.IDSTATUS     <> 'C')                              AND
                                       (NF.IDSISTEMA    = 'PDV')                             AND
                                       (NF.CDFILIAL     = IP.CDFILIAL)                       AND
                                       (NF.NRLANCTONF   = IP.NRLANCTONF)                     AND
                                       (IP.IDCONFORITEM = 'N')                               AND
                                       ((NF.NRAPFATUTNF IS NULL) OR (NF.NRAPFATUTNF = P_NRAPURACFAT)));
   END IF;

 /*VERIFICA SE EXISTE REGISTRO NA TABELA NOVOCODIGO P/ A SRIE DO MRDP*/
   SELECT TO_NUMBER(NVL(MAX(NRSEQUENCIAL),0))
   INTO V_PRXMRDP
   FROM NOVOCODIGO
   WHERE CDCONTADOR = 'LANCAMENTO DE MRDP';

   /*INCLUI CHAVE SEQUENCIAL PARA A SRIE DO MRDP*/
   IF V_PRXMRDP = 0 THEN
       INSERT INTO NOVOCODIGO
         (CDCONTADOR, NRSEQUENCIAL)
      VALUES
         ('LANCAMENTO DE MRDP', '00000000000000000001');

      V_PRXMRDP := 1;
   END IF;


    /*BLOQUEIA REGISTRO DA SRIE DO MRDP NA TABELA NOVOCODIGO*/
   SELECT TO_NUMBER(NRSEQUENCIAL) + 1
   INTO V_PRXMRDP
   FROM NOVOCODIGO
   WHERE CDCONTADOR = 'LANCAMENTO DE MRDP';

   /*INCLUSO DOS MRDP'S APURADOS CONSIDERANDO O NMERO MXIMO DE PRODUTOS MRDP*/

   FOR C_GERAMRDPAPUR IN C_NOTAFISCAL LOOP
        INSERT INTO MRDPAPUR
           (CDFILIAL, NRAPURACFAT, NRMRDPAPUR)
        VALUES
           (P_CDFILIPROC, P_NRAPURACFAT, LPAD(TO_CHAR(V_PRXMRDP),7,'0'));

        V_PRXMRDP := V_PRXMRDP + 1;


   END LOOP;


   /*ATUALIZAO DA TABELA NOVOCODIGO*/
   UPDATE NOVOCODIGO
   SET NRSEQUENCIAL = LPAD(TO_CHAR(V_PRXMRDP),20,'0')
   WHERE CDCONTADOR = 'LANCAMENTO DE MRDP';

   SELECT MAX(CDSERIE), MAX(CDINSCESTA),
          MAX(CDINSCMUNI), MAX(CDEMPRESA)
   INTO V_CDSERIE, V_INSCESTA, V_INSCMUNI, V_CDEMPRESA
   FROM FILIAL F
   WHERE F.CDFILIAL  = P_CDFILIAPUR;

   /*VERIFICA A QTE MAXIMA DE PRODUTO POR SERIE*/
   SELECT NVL(MAX(NRMAXITNF),10)
   INTO V_MAXITENSNF
   FROM SERIENF
   WHERE CDEMPRESA  = V_CDEMPRESA  AND
         CDINSCESTA = V_INSCESTA  AND
         CDINSCMUNI = V_INSCMUNI  AND
         CDSERIE    = V_CDSERIE;


   FOR C_NOTAFISCAL IN C_DELETANF LOOP
      DELETA_NF(C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF);
   END LOOP;

   FOR C_GERANFSAIDA IN C_ITEMNOTA LOOP
      IF V_GERANF = 'S' THEN
         /*INCLUSO DO CORPO DA NOTA*/
         NOVO_CODIGO('LANCNF/FILIAL' || P_CDFILIPROC,V_LANCAMENTO);
         V_LANCAMENTO := SUBSTR(V_LANCAMENTO,11,10);

         GERA_NOTAFISCAL_CORPO(P_CDFILIPROC, V_CDFILIAPUR, V_CDEMPRESA, '01',
                               V_CDSERIE, P_CDCLIENTE, P_DTFINAPUR, 'FAI',
                               V_LANCAMENTO, V_NRLANCESTQ, 'N', P_NRAPURACFAT,
                               V_LANCAMENTO, NULL, NULL, NULL, NULL, NULL, NULL,
                               NULL, NULL, NULL,   NULL, NULL, NULL, NULL,
                               P_DATAPROC, NULL);
         V_GERANF := 'N';
      END IF;

      IF V_MAXITENSNF > V_NRITEM THEN
       /*INCLUSO DOS ITENS DA NOTA FISCAL*/

        GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_LANCAMENTO, C_GERANFSAIDA.CDPRODUTO,
                                C_GERANFSAIDA.CDPRODUTO, C_GERANFSAIDA.NMPRODUTO, C_GERANFSAIDA.QTITEMPROD ,
                                C_GERANFSAIDA.VRUNITITEM, C_GERANFSAIDA.VRDESCITEM, NULL, NULL, NULL, NULL, NULL, NULL);

      /*EFETUA O FECHAMENTO DA NOTA, GERACAO DE IMPOSTOS*/
        GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_LANCAMENTO);

        V_NRITEM := V_NRITEM + 1;
      ELSE

         NOVO_CODIGO('LANCNF/FILIAL' || P_CDFILIPROC,V_LANCAMENTO);
         V_LANCAMENTO := SUBSTR(V_LANCAMENTO,11,10);

         GERA_NOTAFISCAL_CORPO(P_CDFILIPROC, V_CDFILIAPUR, V_CDEMPRESA, '01',
                               V_CDSERIE, P_CDCLIENTE, P_DTFINAPUR, 'FAI',
                               V_LANCAMENTO, V_NRLANCESTQ, 'N', P_NRAPURACFAT,
                               V_LANCAMENTO,NULL, NULL, NULL, NULL, NULL, NULL,
                               NULL, NULL,  NULL,  NULL, NULL, NULL, NULL,
                               P_DATAPROC, NULL);
     /*INCLUSO DOS ITENS DA NOTA FISCAL*/
        GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_LANCAMENTO, C_GERANFSAIDA.CDPRODUTO,
                                C_GERANFSAIDA.CDPRODUTO, C_GERANFSAIDA.NMPRODUTO, C_GERANFSAIDA.QTITEMPROD,
                                C_GERANFSAIDA.VRUNITITEM, C_GERANFSAIDA.VRDESCITEM, NULL, NULL, NULL, NULL,
                                NULL, NULL);

      /*EFETUA O FECHAMENTO DA NOTA, GERACAO DE IMPOSTOS*/
        GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_LANCAMENTO);
        V_NRITEM := 0;
      END IF;
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('APURAC_MULTPRODUTO','4.05.123'); END;
/

