/* Tabela OPERADOR */
/* Descrio: A trigger insere registros na tabela LOGOPERAESPE a partir de alteraes na tabela OPERADOR */
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela OPERADOR
             utilizando a tabela Geral de Logs de Operaes especficas*/
/* Criao:
/* 25/05/2007 - Fabrcio Marcelo Silva */


CREATE OR REPLACE TRIGGER SDXLOG_CTRLACES
AFTER INSERT OR DELETE ON OPERADOR FOR EACH ROW
DECLARE
   V_IDUTLOGCTRLACES  VARCHAR2(1);
   V_IDTIPOOPER       VARCHAR2(1);  /* V_IDTIPOOPER - 'I' - Incluso, 'E' - Excluso */
   V_DESC             VARCHAR2(500);
BEGIN
   BEGIN
      SELECT IDUTLOGCTRLACES
      INTO V_IDUTLOGCTRLACES
      FROM PARAMGERAL
      WHERE CDPARGERAL = '0001';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDUTLOGCTRLACES := 'N';
   END;

   IF V_IDUTLOGCTRLACES = 'S' THEN

      V_IDTIPOOPER  := '';
      V_DESC        := '';

      IF INSERTING THEN

         V_DESC := 'Operador:'        || ' ' || :NEW.CDOPERADOR || '-' || :NEW.NMOPERADOR;

         V_IDTIPOOPER := 'I';

      ELSE

         V_DESC := 'Operador:'        || ' ' || :OLD.CDOPERADOR || '-' || :OLD.NMOPERADOR;

         V_IDTIPOOPER := 'E';
      END IF;

      IF NOT (V_DESC IS NULL) THEN

         V_DESC := SUBSTR(V_DESC, 1, 500);

         INSERT INTO LOGOPERAESPE (NMOPERAESPE,     DTOPERAESPE,     IDOPERAESPE,
                                   CDOPERRESPOPERA, NMOPERRESPOPERA, DSOPERAESPE)
                     VALUES ('Controle Acesso', SYSDATE, V_IDTIPOOPER,
                             '',                '',      V_DESC);
      END IF;
   END IF;
END;
/


/* Grupo de Operador por Operador */
CREATE OR REPLACE TRIGGER SDXLOG_CTRLACESGRU
AFTER INSERT OR DELETE ON OPERGRUPOP FOR EACH ROW
DECLARE
   V_IDUTLOGCTRLACES  VARCHAR2(1);
   V_IDTIPOOPER       VARCHAR2(1);  /* V_IDTIPOOPER - 'I' - Incluso, 'E' - Excluso */
   V_CLIENT_INFO      VARCHAR2(64);
   V_CDOPERADOR       VARCHAR2(12);
   V_DESC             VARCHAR2(500);
   V_NMOPERALT        VARCHAR2(40);
BEGIN
   BEGIN
      SELECT IDUTLOGCTRLACES
      INTO V_IDUTLOGCTRLACES
      FROM PARAMGERAL
      WHERE CDPARGERAL = '0001';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDUTLOGCTRLACES := 'N';
   END;

   IF V_IDUTLOGCTRLACES = 'S' THEN

      V_IDTIPOOPER  := '';
      V_CLIENT_INFO := '';
      V_CDOPERADOR  := '';
      V_DESC        := '';
      V_NMOPERALT   := '';

      /* Retorna o codigo e o nome do operador */
      DBMS_APPLICATION_INFO.READ_CLIENT_INFO(V_CLIENT_INFO);
      V_CDOPERADOR := SUBSTR(V_CLIENT_INFO,1,12);

      IF INSERTING THEN

         BEGIN
            SELECT NMOPERADOR
            INTO V_NMOPERALT
            FROM OPERADOR
            WHERE CDOPERADOR = :NEW.CDOPERADOR;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NMOPERALT := '';
         END;

         V_DESC := 'Operador:'       || ' ' || :NEW.CDOPERADOR || '-' || V_NMOPERALT  || ' ' ||
                   'Grupo Operador:' || ' ' || :NEW.CDGRUPOPER;

         V_IDTIPOOPER := 'I';
      ELSE

         BEGIN
            SELECT NMOPERADOR
            INTO V_NMOPERALT
            FROM OPERADOR
            WHERE CDOPERADOR = :OLD.CDOPERADOR;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NMOPERALT := '';
         END;

         V_DESC := 'Operador:'       || ' ' || :OLD.CDOPERADOR || '-' || V_NMOPERALT  || ' ' ||
                   'Grupo Operador:' || ' ' || :OLD.CDGRUPOPER;

         V_IDTIPOOPER := 'E';
      END IF;

      IF NOT (V_DESC IS NULL) THEN

         V_DESC := SUBSTR(V_DESC, 1, 500);

         INSERT INTO LOGOPERAESPE (NMOPERAESPE,     DTOPERAESPE,     IDOPERAESPE,
                                   CDOPERRESPOPERA, DSOPERAESPE)
                     VALUES ('Grupo Acesso Operador', SYSDATE,      V_IDTIPOOPER,
                             V_CDOPERADOR, V_DESC);
      END IF;
   END IF;
END;
/


/* Filiais por Operador */
CREATE OR REPLACE TRIGGER SDXLOG_CTRLACESFILI
AFTER INSERT OR DELETE ON FILIOPER FOR EACH ROW
DECLARE
   V_IDUTLOGCTRLACES  VARCHAR2(1);
   V_IDTIPOOPER       VARCHAR2(1);  /* V_IDTIPOOPER - 'I' - Incluso, 'E' - Excluso */
   V_CLIENT_INFO      VARCHAR2(64);
   V_CDOPERADOR       VARCHAR2(12);
   V_CDFILIAL         VARCHAR2(4);
   V_DESC             VARCHAR2(500);
   V_NMOPERALT        VARCHAR2(40);
BEGIN
   BEGIN
      SELECT IDUTLOGCTRLACES
      INTO V_IDUTLOGCTRLACES
      FROM PARAMGERAL
      WHERE CDPARGERAL = '0001';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDUTLOGCTRLACES := 'N';
   END;

   IF V_IDUTLOGCTRLACES = 'S' THEN

      V_IDTIPOOPER  := '';
      V_CLIENT_INFO := '';
      V_CDOPERADOR  := '';
      V_CDFILIAL    := '';
      V_DESC        := '';
      V_NMOPERALT   := '';

      /* Retorna o codigo e o nome do operador */
      DBMS_APPLICATION_INFO.READ_CLIENT_INFO(V_CLIENT_INFO);
      V_CDOPERADOR := SUBSTR(V_CLIENT_INFO,1,12);

      IF INSERTING THEN

         BEGIN
            SELECT NMOPERADOR
            INTO V_NMOPERALT
            FROM OPERADOR
            WHERE CDOPERADOR = :NEW.CDOPERADOR;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NMOPERALT := '';
         END;

         V_DESC := 'Operador:'  || ' ' || :NEW.CDOPERADOR || '-' || V_NMOPERALT  || ' ' ||
                   'Filial:'    || ' ' || :NEW.CDFILIAL;

         V_IDTIPOOPER := 'I';

         V_CDFILIAL   := :NEW.CDFILIAL;
      ELSE

         BEGIN
            SELECT NMOPERADOR
            INTO V_NMOPERALT
            FROM OPERADOR
            WHERE CDOPERADOR = :OLD.CDOPERADOR;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NMOPERALT := '';
         END;

         V_DESC := 'Operador:'  || ' ' || :OLD.CDOPERADOR || '-' || V_NMOPERALT  || ' ' ||
                   'Filial:'    || ' ' || :OLD.CDFILIAL;

         V_IDTIPOOPER := 'E';

         V_CDFILIAL   := :OLD.CDFILIAL;
      END IF;

      IF NOT (V_DESC IS NULL) THEN

         V_DESC := SUBSTR(V_DESC, 1, 500);

         INSERT INTO LOGOPERAESPE (NMOPERAESPE,     DTOPERAESPE,     IDOPERAESPE,
                                   CDOPERRESPOPERA, DSOPERAESPE, CDFILIRESPOPERA)
                     VALUES ('Filial Acesso Operador', SYSDATE,      V_IDTIPOOPER,
                             V_CDOPERADOR,  V_DESC, V_CDFILIAL);
      END IF;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('SDXLOG_CTRLACESFILI','5.05.021'); END;
/

